"""
This module contains logging utilities for LDAPStorage. It only logs
messages of severity equal or lower than 'INFO' when in debug mode.
"""

import logging
import config

logger = logging.getLogger('LDAPStorage')

# generic log method
def log(message, summary='', severity=logging.INFO):
    if severity > logging.INFO:
        logger.log(severity, '%s \n%s', summary, message)
    elif config.DEBUG_MODE:
        logger.log(severity, '%s \n%s', summary, message)



"""
An example content type using LDAPStorage.
"""

from Products.Archetypes import public as atapi
from archetypes.ldapstorage import LDAPStorage
from archetypes.ldapstorage import config

attr_mapping = {
    # at field: ldap attr
    'name': 'cn',
    'surname': 'sn',
}

def get_storage(field_id):
    return LDAPStorage(
        'cn',
        'name',
        attr_mapping[field_id],
        default_attrs = {
            'objectClass': ['inetOrgPerson'], 
        },
        required_attrs = ['sn', 'cn'],
    )
    

schema = atapi.BaseSchema + atapi.Schema((
    atapi.StringField(
        'name',
        #default = 'My Name',
        storage=get_storage('name'),
    ),
    atapi.StringField(
        'surname',
        #default = 'My Surname',
        storage=get_storage('surname'),
    ),
))

class ExampleContent2(atapi.BaseContent):
    """ Another example content type using LDAPStorage.
    """
    portal_type = meta_type = 'ExampleContent2'
    archetype_name = 'Example Content 2'

    schema = schema

    _at_rename_after_creation = True

atapi.registerType(ExampleContent2, config.PROJECT_NAME)



"""
An example content type using LDAPStorage.
"""

from Products.Archetypes import public as atapi
from archetypes.ldapstorage import LDAPStorage
from archetypes.ldapstorage import config

attr_mapping = {
    # at field: ldap attr
    'lastname': 'sn',
    'address': 'homePostalAddress',
    'photo': 'jpegPhoto',
}

def get_storage(field_id):
    return LDAPStorage(
        'uid',
        'getId',
        attr_mapping[field_id],
        default_attrs = {
            'objectClass': ['inetOrgPerson'], 
            'cn': ['test name'],
        },
        required_attrs = ['sn', 'cn'],
    )

schema = atapi.BaseSchema + atapi.Schema((
    atapi.StringField(
        'lastname',
        default = 'Test lastname',
        required = 1,
        # It is also required in LDAP, so it must have a default value.
        storage = get_storage('lastname'),
    ),
    atapi.TextField(
        'address',
        storage = get_storage('address'),
    ),
    atapi.ImageField(
        'photo',
        storage = get_storage('photo'),
    ),
))


class ExampleContent(atapi.BaseContent):
    """ An example content type using LDAPStorage.
    """

    portal_type = meta_type = 'ExampleContent'
    archetype_name = 'Example Content'

    schema = schema

    _at_rename_after_creation = True

atapi.registerType(ExampleContent, config.PROJECT_NAME)



#  -*- coding: utf8 -*-
#  Copyright (c) 2007 Eurotux Informática S.A.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
"""
An Archetypes storage that stores the field values in an LDAP directory.
"""

__author__  = 'Ricardo Alves <rsa at eurotux dot com>'
__docformat__ = 'plaintext'

from storage import LDAPStorage
import config
from Products.CMFCore import utils as cmfutils
from Products.Archetypes import public as atapi
from Products.CMFCore import permissions

def initialize(context):

    #if config.DEBUG_MODE:
    import examples.ExampleContent
    import examples.ExampleContent2

    project_name = config.PROJECT_NAME

    # Process the project's types
    content_types, constructors, ftis = \
        atapi.process_types(atapi.listTypes(project_name), project_name)

    cmfutils.ContentInit(
        meta_type           = project_name + ' Content',
        content_types       = content_types,
        permission          = permissions.AddPortalContent,
        extra_constructors  = constructors,
        fti                 = ftis,
    ).initialize(context)



"""
Configuration of LDAPConnection unit tests.
"""

LDAP_SETTINGS = {
    'url': 'ldap://localhost',
    'base_dn': 'ou=people,dc=localhost,dc=localdomain',
    'bind_dn': 'cn=Manager,dc=localhost,dc=localdomain',
    'bind_pwd': 'test',
    'network_timeout': 5,
    'op_timeout': -1,}



#  -*- coding: utf8 -*-
#  Copyright (c) 2007 Eurotux Informática S.A.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
"""
An Archetypes storage that stores field values in LDAP directory.
"""

__author__  = 'Ricardo Alves <rsa at eurotux dot com>'
__docformat__ = 'plaintext'

from Products.CMFCore.utils import getToolByName
from Products.Archetypes.config import TOOL_NAME
from Products.Archetypes.Storage import StorageLayer
from Products.Archetypes.Registry import registerStorage
from Products.Archetypes.interfaces.storage import ISQLStorage
from Products.Archetypes.interfaces.layer import ILayer
from Products.Archetypes.public import AttributeStorage, BaseUnit
from Products.Archetypes.utils import shasattr
from Products.Archetypes.Field import File, Image
from exceptions import MissingRDNError, InvalidRDNError
from DateTime import DateTime
from random import random
import ldap
from log import log

_marker = ''


class LDAPStorage(StorageLayer, AttributeStorage):
    """ An Archetypes storage that stores field values in LDAP directory.
    """

    # FIXME: implementing ISQLStorage (marker interface from Archetypes) to
    # allow the mapping with content types in archetype_tool.
    __implements__ = (ISQLStorage, ILayer)

    def __init__(self, rdn_attr, rdn_map, attr, default_attrs={},
        required_attrs=[]):
        # LDAP rdn attribute
        self._rdn_attr = rdn_attr
        # AT field mapped to rdn. This field shall not me stored with 
        # this storage.
        self._rdn_map = rdn_map
        # Attribute to store
        self._attr = attr
        # The values in default_attrs will be used in initialization
        self._default_attrs = default_attrs
        # Required attributes in the LDAP schema used.
        self._required_attrs = required_attrs

    def getDN(self, instance):
        """ Returns the dn for a specific instance.
        """
        connection = self._getConnection(instance)
        return '%s=%s,%s' % (self._rdn_attr, 
            self.getRDN(instance), connection._base_dn)

    def getRDN(self, instance):
        """ Return the value of rdn for a specific instance.
        Uses the key defined in '_rdn_map' to get the value.
        The key '_rdn_map' may be a callable method, an attribute or an 
        AT field.
        """
        if not self.is_initialized(instance):
            # not initialized yet. if the RDN is an AT Field, we need to get
            # a value to use during initialization.
            f = instance.getField(self._rdn_map)
            if f and f.getStorage().__class__ == self.__class__:
                # it is an AT field and uses LDAP storage.
                if f.getStorage()._attr != self._rdn_attr:
                    # check definition to avoid bugs that are really 
                    # hard to track.
                    raise InvalidRDNError, ("Invalid RDN definition: "
                        "'rdn_attr' must be the same as 'attr' for "
                        "an LDAP-stored RDN map.")
                value  = self.getDefaultValue(instance, self._rdn_map)
                if value:
                    instance._ldap_rdn_value = value
                    instance._p_changed = 1
                    return value

        if hasattr(instance.aq_base, self._rdn_map):
            method = getattr(instance, self._rdn_map)
            if callable(method):
                return method()
            else:
                return method
        else:
            f = instance.getField(self._rdn_map)
            if f and f.getStorage().__class__ == self.__class__:
                if shasattr(instance, '_ldap_rdn_value'):
                    return instance._ldap_rdn_value
        raise MissingRDNError, 'Unable to obtain RDN.'

    def prepare_value(self, instance, name, value):
        """ Prepare value to write in LDAP.
        """
        if not value:
            return ''

        def decode_value(value):
            if type(value) != type(u''):
                try:
                    # FIXME: should we care?
                    return value.decode(instance.getCharset())
                except UnicodeDecodeError:
                    # FIXME: should we let it go?
                    pass
            return value

        if isinstance(value, BaseUnit):
            value = [value.getRaw()]

        if type(value) in (File, Image):
            value = value.data

        if type(value) not in (type([]), type(())):
            value = [value]

        value = [decode_value(v) for v in value]

        field = instance.getField(name)
        f_storage = field.getStorage()

        if self._default_attrs.has_key(f_storage._attr):
            default_value = self._default_attrs[f_storage._attr]
            if type(default_value) == type(''):
                # default value must be an attribute (maybe callable) in
                # instance
                if shasattr(aq_base(instance), default_value):
                    method = getattr(instance, default_value)
                    if callable(method):
                        default_value = [method()]
                    else:
                        default_value = [method]

            value = default_value + \
                [v for v in value if v not in default_value]

        return value

    def getDefaultValue(self, instance, name):
        """ Returns the default value for a specific field.
        """
        field = instance.getField(name)

        try:
            # Try to get a value from the request
            f_value = instance.REQUEST.get(field.getName(),
                field.getDefault(instance))
        except AttributeError:
            f_value = field.getDefault(instance)

        if (not f_value) and (field.getStorage()._attr in \
            self._required_attrs or name == self._rdn_map):
            # this is a required attribute in LDAP schema, or the field
            # mapped to RDN attribute. Use a temporary/random value. 
            # FIXME: this only works for text fields.
            # TODO: document this feature!
            if self._rdn_map == name and shasattr(instance, '_ldap_rdn_value'):
                # If the rdn was already calculated return the same value.
                return instance._ldap_rdn_value
            now=DateTime()
            time = '%s.%s' % \
                (now.strftime('%Y-%m-%d'), str(now.millis())[7:])
            rand = str(random())[2:6]
            f_value = name + time + rand
        return f_value


    def getDefaultValues(self, instance):
        """ Returns a dict with default values for LDAP attributes to use in
        layer initialization.
        """
        ldap_fields = [f for f in instance.Schema().fields() if \
            f.getStorage().__class__ == self.__class__]

        attribute_dict = {}
        for field in ldap_fields:
            f_storage = field.getStorage()
            f_value = self.getDefaultValue(instance, field.getName())
            attribute_dict[f_storage._attr] = \
                self.prepare_value(instance, field.getName(), f_value)

        for key in self._default_attrs.keys():
            if key not in attribute_dict.keys():
                attribute_dict[key] = self._default_attrs[key]
            # TODO: should we concatenate default_attrs and default_values
            # if already defined?

        if not attribute_dict.has_key(self._rdn_attr):
            # add rdn if not there
            attribute_dict[self._rdn_attr] = self.getRDN(instance)

        return attribute_dict


    def is_initialized(self, instance):
        """ Test if the storage layer is already initialized in instance.
        """
        try:
            return self.getName() in instance._initializedStorage
        except AttributeError:
            return False

    def is_cleaned(self, instance):
        """ Test if the storage layer is cleaned in instance.
        """
        try:
            return self.getName() in instance._cleanedStorage
        except AttributeError:
            return False

    def entry_exists(self, instance, dn=None):
        """ Test if the entry already exists.
        """
        if dn is None:
            dn = self.getDN(instance)
        connection = self._getConnection(instance)
        return bool(connection.search(dn))


    def initializeInstance(self, instance, item=None, container=None,
        swallow_exceptions=True):
        """ Implementation of 'initializeInstance' for StorageLayer.
        """

        is_new_object = not getattr(instance, '_v_old_rdn', None)
        
        if instance.isTemporary():
            # It is a temporary object from portal factory. Don't use LDAP
            # yet.
            log('Is a temporary object from portal factory. Dont use LDAP')
            return

        if self.is_initialized(instance) or \
            getattr(instance, '_at_is_fake_instance', _marker):
            # Already intialized or fake instance
            log('Already intialized or fake instance. not initializing')
            return

        dn = self.getDN(instance)

        if self.entry_exists(instance, dn) and is_new_object:
            # Entry already exists.
            log('Entry already exists.  not initializing')
            return
            
        log('initializing %s' % dn)

        # construct attribute_list with current values of fields using 
        # this storage.
        attribute_list = self.getDefaultValues(instance)
        log('Adding with default values: ' + repr(attribute_list))

        connection = self._getConnection(instance)

        if is_new_object:
            # This is really a new object.
            try:
                # create new entry
                connection.addEntry(dn, attribute_list)
            except ldap.OBJECT_CLASS_VIOLATION, e:
                # couldn't initialize (add) due to schema violation.
                log("couldn't initialize due to schema violation: \n %s" % e)
                if not swallow_exceptions:
                    raise e
                # initialization failed but ignore it.
                # initialization will be called again by field mutators, but
                # without swallow_exceptions.
                return
        else:
            # This object was moved. Perform rename operation if RDN is
            # different.
            old_rdn_value = instance._v_old_rdn
            new_rdn_value = self.getRDN(instance)
            if old_rdn_value != new_rdn_value:
                # rdn changed. perform rename operation
                old_dn = u'%s=%s,%s' % (self._rdn_attr, old_rdn_value,
                    connection._base_dn)
                new_rdn = u'%s=%s' % (self._rdn_attr, new_rdn_value)
                connection.modifyDN(old_dn, new_rdn)

        # mark initialized
        try:
            instance._initializedStorage += (self.getName(),)
        except AttributeError:
            instance._initializedStorage = (self.getName(),)

        log('initialized %s' % dn)

        try:
            del instance._cleanedStorage
        except AttributeError:
            pass


    def _getConnection(self, instance):
        """ Return the connection object to be used by this storage.
        """
        at_tool = getToolByName(instance, TOOL_NAME)
        connection_id = at_tool.getConnFor(instance)
        return getattr(instance, connection_id) 


    def set(self, name, instance, value, **kwargs):
        """ Mutator for field value. What may happen:

        * there is no value: do nothing;

        * this field is the rdn map:

            - the instance is not initialized: store the value in
            self._ldap_rdn_value

            - the instance is initialized: store the value in
            self._ldap_rdn_value and in LDAP

            - if the value changes: perform rename operation in LDAP;

        * this is not a field in AT schema (probably an image scale): use
        AttributeStorage to store the value.
            
        """

        log('ldapstorage: mutator for %s' % name)

        if not value:
            return

        connection = self._getConnection(instance)

        if name == self._rdn_map:
            # This field is used to map the LDAP RDN attribute.
            # The value must also be stored in the ZODB.
            # If this value changes, we need to perform a rename in
            # LDAP
            log('ldapstorage: %s is rdn_map, store value' % name)
            if instance._ldap_rdn_value != value:
                # RDN changed. need to perform a rename
                log('ldapstorage: RDN changed, renaming')
                old_dn = u'%s=%s,%s' % (self._rdn_attr,
                    instance._ldap_rdn_value, connection._base_dn)
                new_rdn = u'%s=%s' % (self._rdn_attr, value)
                results = connection.search(old_dn)
                log('ldapstorage: ' + repr(results))
                connection.modifyDN(old_dn, new_rdn)
            # store value in the ZODB
            instance._ldap_rdn_value = value
            instance._p_changed = 1
        
        if not self.is_initialized(instance):
            # not initialized. try to initialize, but without
            # swallow_exceptions.
            log('not initialized. try to initialize')
            self.initializeInstance(instance, swallow_exceptions=False)

        # It is initialized. Do the storage work.

        if not instance.getField(name):
            # Not a field in AT schema. It is probably an image scale.
            # Use AttributeStorage instead.
            return AttributeStorage.set(self, name, instance, value, **kwargs)

        dn = self.getDN(instance)

        value = self.prepare_value(instance, name, value)

        connection.modifyEntry(dn, {self._attr: value})


    def unset(self, name, instance, **kwargs):
        if not instance.getField(name):
            # Not a field in AT schema. It is probably an image scale.
            # Use AttributeStorage instead.
            return AttributeStorage.unset(self, name, instance, **kwargs)

        connection = self._getConnection(instance)
        dn = self.getDN(instance)

        connection.modifyEntry(dn, {self._attr: []})

    def get(self, name, instance, **kwargs):
        """ Accessor for field value. What may happen:

        * instance is not initialized: there is nothing to get;

        * it fails to retrieve the DN:

            - if this field is the RDN map, return the rdn value. else
            raise exception

        * we have a dn: get the value (a list):

            - check if this field is multivalued, or lines, to return a list
              or only the first element.
            
        """

        if name == self._rdn_map:
            # This field is used to map the LDAP RDN attribute.
            # Return the value stored in the ZODB.
            if shasattr(instance, '_ldap_rdn_value'):
                return instance._ldap_rdn_value

        if not self.is_initialized(instance):
            log("not initialized: don't get, but return default value")
            return instance.getField(name).getDefault(instance)

        connection = self._getConnection(instance)
        dn = self.getDN(instance)

        log('search dn: ' + dn)
        results = connection.search(dn, ldap.SCOPE_BASE, attrlist=[self._attr])
        log('search results: ' + repr(results))

        if results:
            log('return: ' + repr(results[0][1].get(self._attr, _marker)))
            value = results[0][1].get(self._attr, _marker)
        else:
            value = _marker

        # handle list and non-list values
        field = instance.getField(name)
        if value and type(value) == type([]):
            if field.type == 'lines' or field.multiValued:
                return value
            else:
                # TODO: we should log a warning message when a 
                # single-valued AT field has multiple values in LDAP
                return value[0]

        return value


    def initializeField(self, instance, field):
        pass


    def cleanupField(self, instance, field):
        pass


    def cleanupInstance(self, instance, item=None, container=None):
        """ Cleanup instance when deleted.
        """
        if self.is_cleaned(instance) or getattr(instance.aq_base,
            '_at_is_fake_instance', None):
            # Already cleaned or fake instance.
            log('Already cleaned or fake instance. dont clean')
            return

        dn = self.getDN(instance)

        log('Cleaning up instance %s' % dn)

        if getattr(instance, '_v_cp_refs', None):
            # Object is beeing moved. Do not remove the entry and mark the
            # old rdn.
            instance._v_old_rdn = self.getRDN(instance)
        else:
            # object is really going away. Remove entry from LDAP.
            conn = self._getConnection(instance)
            conn.deleteEntry(dn)
            

        # marked cleaned
        try:
            instance._cleanedStorage += (self.getName(),)
        except AttributeError:
            instance._cleanedStorage = (self.getName(),)

        log('Cleaned instance %s' % dn)

        try:
            del instance._initializedStorage
        except AttributeError:
            pass


registerStorage(LDAPStorage)



#  -*- coding: utf8 -*-
"""
Test suit for LDAPStorage.
"""

__author__  = 'Ricardo Alves <rsa at eurotux dot com>'
__docformat__ = 'plaintext'

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from archetypes.ldapstorage import config as product_config
product_config.DEBUG_MODE = 1

import config as tests_config

import transaction
from Globals import InitializeClass, package_home
from Testing import ZopeTestCase
from Products.PloneTestCase import PloneTestCase
from Products.CMFPlone.utils import base_hasattr
from Products.PloneTestCase.layer import onsetup
from Products.Five import zcml
from Products.ldapconnection import LDAPConnection
import ldap
import os.path

ZopeTestCase.installProduct('ldapconnection')

@onsetup
def setup_package():
    import archetypes.ldapstorage
    zcml.load_config('configure.zcml', archetypes.ldapstorage)
    ZopeTestCase.installPackage('archetypes.ldapstorage')

setup_package()

PloneTestCase.setupPloneSite(extension_profiles=['archetypes.ldapstorage:example'])

PACKAGE_HOME = package_home(globals())

class TestLDAPStorage(PloneTestCase.PloneTestCase):
    """ Test suit for LDAPStorage.
    """

    def afterSetUp(self):
        self.setupConnection()

    def setupConnection(self):
        """ Create and configure an LDAP connection.
        """
        # create new connection
        self.portal.manage_addProduct['ldapconnection'].\
            manage_addLDAPConnection('ldap_connection',
                'Test LDAP Connection', **tests_config.LDAP_SETTINGS)
        # set connection for portal types
        self.portal.archetype_tool.setConnForPortalTypes(
            ['ExampleContent', 'ExampleContent2'], 'ldap_connection')

    def test_contentCreation(self):
        self.folder.invokeFactory(id='content1', type_name='ExampleContent')
        self.failUnless(base_hasattr(self.folder, 'content1'))

        # search LDAP for created entry
        conn = self.portal.ldap_connection
        results = conn.search('uid=content1,%s' % \
            tests_config.LDAP_SETTINGS['base_dn'])
        self.failUnless(results)

        self.folder.manage_delObjects(['content1'])

    def test_variableRDN(self):
        self.folder.invokeFactory(id='contentb', type_name='ExampleContent2')
        self.failUnless(base_hasattr(self.folder, 'contentb'))
        obj = self.folder.contentb

        # test RDN value
        rdn_value = obj._ldap_rdn_value
        self.failUnlessEqual(rdn_value.encode(obj.getCharset()), obj.getName())

        # search LDAP for created entry
        conn = self.portal.ldap_connection
        results = conn.search('cn=%s,%s' % (rdn_value, 
            tests_config.LDAP_SETTINGS['base_dn']))
        self.failUnless(results)

        # compare rdn value
        self.failUnlessEqual(rdn_value, results[0][1]['cn'][0])

        # test edition
        obj.edit(name='Césare', surname='Your surname')

        self.failUnlessEqual('Césare', obj.getName())
        self.failUnlessEqual('Your surname', obj.getSurname())

        # test RDN value again
        rdn_value = obj._ldap_rdn_value
        self.failUnlessEqual(rdn_value.encode(obj.getCharset()), obj.getName())

        results = conn.search('cn=%s,%s' % (rdn_value,
            tests_config.LDAP_SETTINGS['base_dn']))

        # compare rdn value
        self.failUnlessEqual(rdn_value.encode(obj.getCharset()), 
            results[0][1]['cn'][0])

        self.folder.manage_delObjects(['contentb'])

    #def test_objectClassViolation(self):
    #    pass

    #def test_invalidRDN(self):
    #    pass

    def test_contentEdit(self):
        self.folder.invokeFactory(id='content1', type_name='ExampleContent')
        obj = getattr(self.folder, 'content1')

        # read image file
        photo_file = open(os.path.join(PACKAGE_HOME, 'input', 'photo.png'), 
            'rb')
        obj.edit(address='15th Avenue, 50', lastname='New Last Name', 
            photo=photo_file.read())

        # test values through accessors
        self.failUnlessEqual(obj.getAddress(), '15th Avenue, 50')
        self.failUnlessEqual(obj.getLastname(), 'New Last Name')

        # test values in LDAP
        conn = self.portal.ldap_connection
        dn = 'uid=content1,%s' % tests_config.LDAP_SETTINGS['base_dn']
        results = conn.search(dn, ldap.SCOPE_BASE)
        self.failUnlessEqual(results[0][1]['homePostalAddress'][0], 
            '15th Avenue, 50')
        self.failUnlessEqual(results[0][1]['sn'][0], 'New Last Name')

        # cleanup
        self.folder.manage_delObjects(['content1'])

        # test if was deleted in LDAP
        self.assertEqual(conn.search(dn, ldap.SCOPE_BASE), [])

    def test_contentMove(self):
        self.folder.invokeFactory(id='content1', type_name='ExampleContent')
        obj = getattr(self.folder, 'content1')

        # pass flag to the transaction savepoint method to indicate
        # that databases without savepoint support should be tolerated.
        transaction.savepoint(1)

        self.folder.manage_renameObject('content1', 'content2')

        old_dn = 'uid=content1,%s' % tests_config.LDAP_SETTINGS['base_dn']
        new_dn = 'uid=content2,%s' % tests_config.LDAP_SETTINGS['base_dn']

        conn = self.portal.ldap_connection
        self.assertEqual(conn.search(old_dn, ldap.SCOPE_BASE), [])

        results = conn.search(new_dn, ldap.SCOPE_BASE)
        self.failUnless(results)

        obj = getattr(self.folder, 'content2')
        self.assertEqual(obj.getLastname(), 'Test lastname')
        self.assertEqual(results[0][1]['sn'][0], 'Test lastname')

        # cleanup
        self.folder.manage_delObjects(['content2'])

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestLDAPStorage))
    return suite

if __name__ == '__main__':
    framework()
